@@ -20,34 +20,34 @@ def set_argument(var, fun_kwargs, base_value):
 
 class Systems_population_constructor(object):
     def __init__(self, pool, terms_number, max_factors_in_term, eq_search_evo, 
-                 obj_functions = None, sparcity_interval = (0, 1)):
+                 obj_functions = None, sparsity_interval = (0, 1)):
         self.pool = pool; self.terms_number = terms_number
         self.eq_search_evo = eq_search_evo
         self.max_factors_in_term = max_factors_in_term; #self.eq_search_iters = eq_search_iters
         self.equation_number = len(self.pool.families_meaningful)
-        self.sparcity_interval = sparcity_interval
+        self.sparsity_interval = sparsity_interval
 #        print(self.equation_number)
 #        raise NotImplementedError
         #len([1 for token_family in self.tokens if token_family.status['meaningful']])
     
     def create(self, **kwargs): # Дописать
         pop_size = set_argument('population_size', kwargs, 16)
-        sparcity = set_argument('sparcity', kwargs, np.power(np.e, np.random.uniform(low = np.log(self.sparcity_interval[0]),
-                                                                      high = np.log(self.sparcity_interval[1]),
+        sparsity = set_argument('sparsity', kwargs, np.power(np.e, np.random.uniform(low = np.log(self.sparsity_interval[0]),
+                                                                      high = np.log(self.sparsity_interval[1]),
                                                                       size = self.equation_number)))
 #        eq_search_iters = set_argument('eq_search_iters', kwargs, 50)
         
-        print(f'Creating new equation, sparsity value {sparcity}')
+        print(f'Creating new equation, sparsity value {sparsity}')
         created_solution = SoEq(pool = self.pool, terms_number = self.terms_number,
                                 max_factors_in_term = self.max_factors_in_term, 
-                                sparcity = sparcity)
+                                sparsity = sparsity)
         try:
             created_solution.set_objective_functions(kwargs['obj_funs'])
         except KeyError:
             created_solution.use_default_objective_function()
         created_solution.set_eq_search_evolutionary(self.eq_search_evo)
 #        print('searching equations with ', eq_search_iters, 'iterations, and popsize of ', pop_size)
-        created_solution.create_equations(pop_size, sparcity)
+        created_solution.create_equations(pop_size, sparsity)
         print('Equation created', type(created_solution))
         return created_solution        
     
